@@ -10,13 +10,13 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = "0.66";
+our $VERSION = "0.73";
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-use Devel::Cover::DB  0.66;
-use Devel::Cover::Inc 0.66;
+use Devel::Cover::DB  0.73;
+use Devel::Cover::Inc 0.73;
 
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
@@ -45,10 +45,12 @@ my $Initialised;                         # import() has been called.
 
 my $Dir;                                 # Directory in which coverage will be
                                          # collected.
-my $DB        = "cover_db";              # DB name.
-my $Merge     = 1;                       # Merge databases.
-my $Summary   = 1;                       # Output coverage summary.
-my $Subs_only = 0;                       # Coverage only for sub bodies.
+my $DB             = "cover_db";         # DB name.
+my $Merge          = 1;                  # Merge databases.
+my $Summary        = 1;                  # Output coverage summary.
+my $Subs_only      = 0;                  # Coverage only for sub bodies.
+my $Self_cover;                          # Coverage of Devel::Cover.
+my $Self_cover_run = 0;                  # Covering Devel::Cover now.
 
 my @Ignore;                              # Packages to ignore.
 my @Inc;                                 # Original @INC to ignore.
@@ -77,6 +79,10 @@ my %Coverage_options;                    # Options for overage criteria.
 
 my %Run;                                 # Data collected from the run.
 
+my $Const_right = qr/^(?:const|s?refgen|gelem|die|undef|bless|anon(?:list|hash)|
+                       scalar|return|last|next|redo|goto)$/x;
+                                         # constant ops
+
 use vars '$File',                        # Last filename we saw.  (localised)
          '$Line',                        # Last line number we saw.  (localised)
          '$Collect',                     # Whether or not we are collecting
@@ -84,7 +90,7 @@ use vars '$File',                        # Last filename we saw.  (localised)
                                          # over conditions.  (localised)
          '%Files',                       # Whether we are interested in files.
                                          # Used in runops function.
-         '$Replace_ops',
+         '$Replace_ops',                 # Whether we are replacing ops.
          '$Silent';                      # Output nothing. Can be used anywhere.
 
 BEGIN
@@ -93,6 +99,12 @@ BEGIN
     $Silent = ($ENV{HARNESS_PERL_SWITCHES} || "") =~ /Devel::Cover/ ||
               ($ENV{PERL5OPT}              || "") =~ /Devel::Cover/;
     *OUT = $ENV{DEVEL_COVER_DEBUG} ? *STDERR : *STDOUT;
+
+    @Inc = @Devel::Cover::Inc::Inc;
+    @Ignore = ("/Devel/Cover[./]") unless $Self_cover = $ENV{DEVEL_COVER_SELF};
+    # $^P = 0x004 | 0x010 | 0x100 | 0x200;
+    # $^P = 0x004 | 0x100 | 0x200;
+    $^P |= 0x004 | 0x100;
 }
 
 if (0 && $Config{useithreads})
@@ -154,11 +166,6 @@ if (0 && $Config{useithreads})
     };
 }
 
-BEGIN { @Inc = @Devel::Cover::Inc::Inc; @Ignore = ("/Devel/Cover[./]") }
-# BEGIN { $^P = 0x004 | 0x010 | 0x100 | 0x200 }
-# BEGIN { $^P = 0x004 | 0x100 | 0x200 }
-BEGIN { $^P = 0x004 | 0x100 }
-
 {
     sub check
     {
@@ -250,7 +257,7 @@ EOM
     POSIX::_exit(1);
 }
 
-$Replace_ops = 1;
+$Replace_ops = !$Self_cover;
 
 sub import
 {
@@ -287,6 +294,11 @@ sub import
         warn __PACKAGE__ . ": Unknown option $_ ignored\n";
     }
 
+    my $ci     = $^O eq "MSWin32";
+    @Select_re = map qr/$_/,                           @Select;
+    @Ignore_re = map qr/$_/,                           @Ignore;
+    @Inc_re    = map $ci ? qr/^\Q$_\//i : qr/^\Q$_\//, @Inc;
+
     bootstrap Devel::Cover $VERSION;
 
     if (defined $Dir)
@@ -304,9 +316,9 @@ sub import
 
     unless (-d $DB)
     {
-        mkdir $DB, 0700 or croak "Can't mkdir $DB: $!\n";
+        mkdir $DB, 0700 or die "Can't mkdir $DB: $!";
     }
-    $DB = $1 if Cwd::abs_path($DB) =~ /(.*)/;
+    $DB = $1 if abs_path($DB) =~ /(.*)/;
     Devel::Cover::DB->delete($DB) unless $Merge;
 
     if ($blib)
@@ -316,10 +328,6 @@ sub import
         push @Ignore, "^t/", '\\.t$', '^test\\.pl$';
     }
 
-    my $ci = $^O eq "MSWin32";
-    @Select_re = map qr/$_/,                           @Select;
-    @Ignore_re = map qr/$_/,                           @Ignore;
-    @Inc_re    = map $ci ? qr/^\Q$_\//i : qr/^\Q$_\//, @Inc;
     %Files     = ();  # start gathering file information from scratch
 
     for my $c (Devel::Cover::DB->new->criteria)
@@ -426,22 +434,30 @@ sub normalised_file
     }
     if ($] >= 5.008)
     {
-        if ($^O eq "MSWin32" || $^O eq "cygwin")
+        my $inc;
+        $inc ||= $file =~ $_ for @Inc_re;
+        # warn "inc for [$file] is [$inc] @Inc_re";
+        if ($inc && ($^O eq "MSWin32" || $^O eq "cygwin"))
         {
-            # TODO - Windows seems busted here
+            # Windows' Cwd::_win32_cwd() calls eval which will recurse back
+            # here if we call abs_path, so we just assume it's normalised.
+            # warn "giving up on getting normalised filename from <$file>\n";
         }
         else
         {
             # print STDERR "getting abs_path <$file> ";
-            my $abs;
-            $abs = abs_path($file) unless -l $file;  # leave symbolic links
-            # print STDERR "giving <$file> ";
-            $file = $abs if defined $abs;
+            if (-e $file)  # Windows likes the file to exist
+            {
+                my $abs;
+                $abs = abs_path($file) unless -l $file;  # leave symbolic links
+                # print STDERR "giving <$abs> ";
+                $file = $abs if defined $abs;
+            }
         }
         # print STDERR "finally <$file> <$Dir>\n";
     }
     $file =~ s|\\|/|g if $^O eq "MSWin32";
-    $file =~ s|^$Dir/||;
+    $file =~ s|^$Dir/|| if defined $Dir;
 
     # print STDERR "File: $f => $file\n";
 
@@ -476,12 +492,16 @@ sub use_file
 {
     my ($file) = @_;
 
+    # warn "use_file($file)\n";
+
     # die "bad file" unless length $file;
 
-    $file = $1 if $file =~ /^\(eval \d+\)\[(.*):\d+\]/;
+    $file = $1 if $file =~ /^\(eval \d+\)\[(.+):\d+\]/;
     $file =~ s/ \(autosplit into .*\)$//;
 
     return $Files{$file} if exists $Files{$file};
+    return 0 if $file =~ /\(eval \d+\)/          ||
+                $file =~ /^\.\.[\/\\]\.\.[\/\\]lib[\/\\](?:Storable|POSIX).pm$/;
 
     my $f = normalised_file($file);
 
@@ -495,9 +515,7 @@ sub use_file
     # system "pwd; ls -l '$file'";
     $Files{$file} = -e $file ? 1 : 0;
     warn __PACKAGE__ . qq(: Can't find file "$file" (@_): ignored.\n)
-        unless $Files{$file} || $Silent || $file =~ /\(eval \d+\)/ ||
-               $file eq "../../lib/Storable.pm" ||
-               $file eq "../../lib/POSIX.pm";
+        unless $Files{$file} || $Silent;
 
     $Files{$file}
 }
@@ -506,10 +524,10 @@ sub check_file
 {
     my ($cv) = @_;
 
-    return unless class($cv) eq "CV";
+    return unless ref($cv) eq "B::CV";
 
     my $op = $cv->START;
-    return unless $op->can("file") && class($op) ne "NULL" && is_state($op);
+    return unless ref($op) eq "B::COP";
 
     my $file = $op->file;
     my $use  = use_file($file);
@@ -601,7 +619,17 @@ sub check_files
     @Subs = map $_->object_2svref, @Cvs if $] >= 5.008001;
 }
 
+my %Seen;
+
 sub report
+{
+    _report();
+    return unless $Self_cover;
+    $Self_cover_run = 1;
+    _report();
+}
+
+sub _report
 {
     local @SIG{qw(__DIE__ __WARN__)};
 
@@ -615,7 +643,7 @@ sub report
 
     my @collected = get_coverage();
     return unless @collected;
-    set_coverage("none");
+    set_coverage("none") unless $Self_cover;
 
     $Run{collected} = \@collected;
     $Structure      = Devel::Cover::DB::Structure->new(base => $DB);
@@ -681,19 +709,22 @@ sub report
         structure => $Structure,
     );
 
-    $DB .= "/runs";
-    unless (-d $DB)
+    my $dbrun = "$DB/runs";
+    unless (-d $dbrun)
     {
-        mkdir $DB, 0700 or croak "Can't mkdir $DB: $!\n";
+        mkdir $dbrun, 0700 or croak "Can't mkdir $dbrun: $!\n";
     }
-    $DB .= "/$run";
+    $dbrun .= "/$run";
 
-    $cover->{db} = $DB;
-
-    print OUT __PACKAGE__, ": Writing coverage database to $DB\n"
+    print OUT __PACKAGE__, ": Writing coverage database to $dbrun\n"
         unless $Silent;
-    $cover->write;
+    $cover->write($dbrun);
     $cover->print_summary if $Summary && !$Silent;
+
+    return if !$Self_cover || $Self_cover_run;
+
+    $cover->delete;
+    delete $Run{vec};
 }
 
 sub add_subroutine_cover
@@ -739,8 +770,6 @@ sub add_statement_cover
         if exists $Coverage->{time} && exists $Coverage->{time}{$key};
 }
 
-my %Seen;
-
 sub add_branch_cover
 {
     return unless $Collect && $Coverage{branch};
@@ -809,10 +838,14 @@ sub add_condition_cover
 
     my $key = get_key($op);
     # print STDERR "Condition cover $$op from $File:$Line\n";
+    # print STDERR "left:  [$left]\nright: [$right]\n";
+    # use Carp "cluck"; cluck("from here");
 
     my $type = $op->name;
+    # print STDERR "type:  [$type]\n";
     $type =~ s/assign$//;
     $type = "or" if $type eq "dor";
+    # print STDERR "type:  [$type]\n";
 
     my $c = $Coverage->{condition}{$key};
 
@@ -827,11 +860,11 @@ sub add_condition_cover
         $name = $r->first->name if $name eq "sassign";
         # TODO - exec?  any others?
         # print STDERR "Name [$name]\n";
-        if ($c->[5] || $name =~
-            /^const|s?refgen|gelem|die|undef|bless|anon(?:list|hash)|scalar$/)
+        if ($c->[5] || $name =~ $Const_right)
         {
             $c = [ $c->[3], $c->[1] + $c->[2] ];
             $count = 2;
+            # print STDERR "Special short circuit\n";
         }
         else
         {
@@ -989,9 +1022,12 @@ sub logop
     my ($op, $cx, $lowop, $lowprec, $highop, $highprec, $blockname) = @_;
     my $left  = $op->first;
     my $right = $op->first->sibling;
+    # print STDERR "left [$left], right [$right]\n";
     my ($file, $line) = ($File, $Line);
+
     if ($cx < 1 && is_scope($right) && $blockname && $self->{expand} < 7)
     {
+        # print STDERR 'if ($a) {$b}', "\n";
         # if ($a) {$b}
         {
             # local $Collect;
@@ -1004,6 +1040,7 @@ sub logop
     }
     elsif ($cx < 1 && $blockname && !$self->{parens} && $self->{expand} < 7)
     {
+        # print STDERR '$b if $a', "\n";
         # $b if $a
         {
             # local $Collect;
@@ -1016,18 +1053,21 @@ sub logop
     }
     elsif ($cx > $lowprec && $highop)
     {
+        # print STDERR '$a && $b', "\n";
         # $a && $b
         {
-            # local $Collect;
+            local $Collect;
             $left  = $self->deparse_binop_left ($op, $left,  $highprec);
             $right = $self->deparse_binop_right($op, $right, $highprec);
         }
+        # print STDERR "left [$left], right [$right]\n";
         add_condition_cover($op, $highop, $left, $right)
             unless $Seen{condition}{$$op}++;
         return $self->maybe_parens("$left $highop $right", $cx, $highprec)
     }
     else
     {
+        # print STDERR '$a and $b', "\n";
         # $a and $b
         {
             # local $Collect;
@@ -1064,12 +1104,19 @@ sub get_cover
 
     # print STDERR "get_cover: <$Sub_name>\n";
     return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK.
+    # return unless length  $Sub_name;  # Only happens with Self_cover, AFAIK.
 
     get_location($start) if $start;
     # print STDERR "[[$File:$Line]]\n";
     # return unless length $File;
     return if length $File && !use_file($File);
 
+    return if !$Self_cover_run && $File =~ /Devel\/Cover/;
+    return if  $Self_cover_run && $File !~ /Devel\/Cover/;
+    return if  $Self_cover_run &&
+               $File =~ /Devel\/Cover\.pm$/ &&
+               $Sub_name eq "import";
+
     # printf STDERR "getting cover for $Sub_name ($start), %x\n", $$cv;
 
     if ($start)
@@ -1250,6 +1297,9 @@ The most appropriate mailing list on which to discuss this module would
 be perl-qa.  Discussion has migrated there from perl-qa-metrics which is
 now defunct.  See L<http://lists.perl.org/showlist.cgi?name=perl-qa>.
 
+The Devel::Cover repository can be found at
+L<http://github.com/pjcj/Devel--Cover>.
+
 =head1 REQUIREMENTS
 
 =over
@@ -1279,6 +1329,14 @@ if you want syntax highlighted HTML reports.
 
 if you want Pod coverage.
 
+=item * L<Test::More>
+
+in order to run the tests
+
+=item * L<Test::Warn>
+
+in order to run some of the tests
+
 =item * L<Test::Differences>
 
 if the tests fail and you would like nice output telling you why.
@@ -1442,7 +1500,7 @@ See the BUGS file.  And the TODO file.
 
 =head1 VERSION
 
-Version 0.66 - 12th April 2010
+Version 0.73 - 2nd October 2010
 
 =head1 LICENCE
 
