@@ -16,7 +16,7 @@ use Storable;
 
 use Devel::Cover::DB;
 
-our $VERSION = "0.66";
+our $VERSION = "0.73";
 our $AUTOLOAD;
 
 sub new
@@ -197,6 +197,8 @@ sub digest
     my $self = shift;
     my ($file) = @_;
 
+    # warn "Opening $file for MD5 digest\n";
+
     my $digest;
     if (open my $fh, "<", $file)
     {
@@ -207,7 +209,7 @@ sub digest
     {
         warn "Devel::Cover: Can't open $file for MD5 digest: $!\n"
             unless lc $file eq "-e";
-        # use Carp; print STDERR Carp::longmess("in " . `pwd`);
+        # require "Cwd"; warn Carp::longmess("in " . Cwd::cwd());
     }
     $digest
 }
@@ -216,6 +218,7 @@ sub get_count
 {
     my $self = shift;
     my ($criterion) = @_;
+    return 0 unless $self->{file};  # TODO - how does this get unset?
     $self->{count}{$criterion}{$self->{file}}
 }
 
@@ -251,7 +254,7 @@ sub write
         $self->{f}{$file}{file} = $file;
         unless ($self->{f}{$file}{digest})
         {
-            warn "Can't find digest for $file";
+            warn "Can't find digest for $file" unless $Devel::Cover::Silent;
             next;
         }
         my $df = "$dir/$self->{f}{$file}{digest}";
@@ -323,7 +326,7 @@ Huh?
 
 =head1 VERSION
 
-Version 0.66 - 12th April 2010
+Version 0.73 - 2nd October 2010
 
 =head1 LICENCE
 
